#include <BossP.h>

#include <Xm/BulletinBP.h>
#include <Xm/BulletinB.h>

static XtResource resources[] =
{
        {       XmNresizeCallback,
                XmCCallback, XmRCallback, sizeof (XtCallbackList),
                XtOffset (XmBossWidget, boss.resize_callback),
                XmRImmediate, (caddr_t) NULL
        },
        {       XmNchangeManagedCallback,
                XmCCallback, XmRCallback, sizeof (XtCallbackList),
                XtOffset (XmBossWidget, boss.change_managed_callback),
                XmRImmediate, (caddr_t) NULL
        },
        {       XmNgeometryManagerCallback,
                XmCCallback, XmRCallback, sizeof (XtCallbackList),
                XtOffset (XmBossWidget, boss.geometry_manager_callback),
                XmRImmediate, (caddr_t) NULL
        },
        {       XmNqueryGeometryCallback,
                XmCCallback, XmRCallback, sizeof (XtCallbackList),
                XtOffset (XmBossWidget, boss.query_geometry_callback),
                XmRImmediate, (caddr_t) NULL
        },
};

/****************************************************************
 *
 *	XmBoss class record
 *
 ****************************************************************/

static void Redisplay();
static void Resize();
static void ChangeManaged();
static XtGeometryResult GeometryManager();
static XtGeometryResult QueryGeometry();
extern void _XmBulletinBoardFocusMoved();

externaldef( xmbossclassrec) XmBossClassRec
                                                      bossClassRec =
{
   {			/* core_class fields      */
      (WidgetClass) &xmBulletinBoardClassRec,	/* superclass         */
      "XmBoss",					/* class_name         */
      sizeof(XmBossRec),			/* widget_size        */
      NULL,					/* class_initialize   */
      NULL,					/* class_part_init    */
      FALSE,					/* class_inited       */
      NULL,					/* initialize         */
      NULL,					/* initialize_hook    */
      _XtInherit,				/* realize            */
      NULL,					/* actions	      */
      0,					/* num_actions	      */
      resources,				/* resources          */
      XtNumber(resources),			/* num_resources      */
      NULLQUARK,				/* xrm_class          */
      TRUE,					/* compress_motion    */
      XtExposeCompressMaximal,			/* compress_exposure  */
      TRUE,					/* compress_enterlv   */
      FALSE,					/* visible_interest   */
      (XtWidgetProc) NULL,	                /* destroy            */
      Resize,					/* resize             */
      (XtExposeProc) Redisplay,			/* expose             */
      (XtSetValuesFunc) NULL,			/* set_values         */
      NULL,					/* set_values_hook    */
      _XtInherit,				/* set_values_almost  */
      NULL,					/* get_values_hook    */
      NULL,					/* accept_focus       */
      XtVersion,				/* version            */
      NULL,					/* callback_private   */
      XtInheritTranslations,                    /* tm_table           */
      (XtGeometryHandler) QueryGeometry,	/* query_geometry     */
      NULL,             	                /* display_accelerator*/
      NULL,		                        /* extension          */
   },

   {		/* composite_class fields */
      (XtGeometryHandler) GeometryManager,	/* geometry_manager   */
      (XtWidgetProc) ChangeManaged,		/* change_managed     */
      _XtInherit,				/* insert_child       */
      _XtInherit,				/* delete_child       */
      NULL,                                     /* extension          */
   },

   {		/* constraint_class fields */
      NULL,					/* resource list        */   
      0,					/* num resources        */   
      0,					/* constraint size      */   
      NULL,					/* init proc            */   
      NULL,             			/* destroy proc         */   
      NULL,					/* set values proc      */   
      NULL,                                     /* extension            */
   },

   {		/* manager_class fields */
      XtInheritTranslations,			/* default translations   */
      NULL,					/* syn_resources      	  */
      0,					/* num_syn_resources 	  */
      NULL,					/* syn_cont_resources     */
      0,					/* num_syn_cont_resources */
      NULL,               			/* parent_process         */
      NULL,					/* extension		  */
   },
   {            /* bulletinBoard class fields */
      FALSE,                                    /*always_install_accelerators*/
      NULL,                                     /* geo_matrix_create */
      _XmBulletinBoardFocusMoved,               /* focus_moved_proc */
      NULL                                      /* extension */
   },
   {		/* boss class fields */     
      NULL					/* extension */
   }
};

externaldef( bosswidgetclass) WidgetClass bossWidgetClass
                                     = (WidgetClass) &bossClassRec ;

static void Redisplay(
	Widget		bb,
	XEvent *        event,
	Region          region)
{
    (*(xmBulletinBoardClassRec.core_class.expose))
		(bb, event, region );
}

/***********************************************************************
 * ChangeManaged - we gained/lost a child
 **********************************************************************/
static void ChangeManaged( XmBossWidget w )
{
    XmBossChangeManagedCallbackStruct cb;

    cb.reason = XmCR_BOSS_CHANGE_MANAGED;
    cb.event = NULL;
    cb.doResize = False;

    XtCallCallbackList( w, w->boss.resize_callback, &cb );

    if ( cb.doResize )
	Resize( w );
}

/***********************************************************************
 * QueryGeometry - what size do we want to be
 **********************************************************************/
static XtGeometryResult QueryGeometry(
    XmBossWidget boss, XtWidgetGeometry *intended, XtWidgetGeometry *desired )
{
    if ( boss->boss.geometry_manager_callback )
    {
	XmBossQueryGeometryCallbackStruct cb;

	cb.reason = XmCR_BOSS_QUERY_GEOMETRY;
	cb.event = NULL;
	cb.intended = intended;
	cb.desired = desired;
	cb.result = XtGeometryYes;

	XtCallCallbackList( boss, boss->boss.resize_callback, &cb );

	return cb.result;
    }
    else
    {
	return (*(xmBulletinBoardClassRec.core_class.query_geometry))
		    (boss, intended, desired);
    }
}

/***********************************************************************
 * GeometryManager - the child wants a new geometry
 **********************************************************************/
static XtGeometryResult GeometryManager(
	Widget child, XtWidgetGeometry *desired, XtWidgetGeometry *allowed )
{
    XmBossWidget boss = (XmBossWidget)XtParent(child);

    if ( boss->boss.geometry_manager_callback )
    {
	XmBossGeometryManagerCallbackStruct cb;
	cb.reason = XmCR_BOSS_CHANGE_MANAGED;
	cb.event = NULL;
	cb.desired = allowed;
	cb.allowed = desired;
	cb.result = XtGeometryNo;

	XtCallCallbackList( boss, boss->boss.resize_callback, &cb );

	return cb.result;
    }
    else
    {
	return (*(xmBulletinBoardClassRec.composite_class.geometry_manager))
		    (child, desired, allowed );
    }
}

/***********************************************************************
 * Resize - we changed size 
 **********************************************************************/
static void Resize( XmBossWidget w )
{
    XmBossResizeCallbackStruct	cb;
    cb.reason = XmCR_BOSS_RESIZE;
    cb.event = NULL;

    XtCallCallbackList( w, w->boss.resize_callback, &cb );

    /* chain to BulletinBoard to handle shadow */
    (*(xmBulletinBoardClassRec.core_class.resize))(w);
}

Widget XmCreateBoss(
            Widget          p,          /*  parent widget       */
            String          name,       /*  widget name         */
            ArgList         args,       /*  arg list    */
            Cardinal        n)          /*  arg count   */
{
    return( XtCreateWidget( name, bossWidgetClass, p, args, n)) ;
}

